@@ -17,7 +17,7 @@ from . import infin_ast
 from datetime import datetime, timedelta
 import time
 import mlflow
-from mlflow import start_run, end_run, log_metric, log_param, log_artifacts, set_experiment, log_artifact, log_param
+from mlflow import start_run, end_run, log_metric, log_param, log_artifacts, set_experiment, log_artifact, set_tag
 from mlflow.tracking.client import MlflowClient
 from mlflow.entities import ViewType
 from os.path import sep
@@ -103,6 +103,28 @@ def get_label_info(label):
     return label['timespec']['S'], label['bucketname']['S'], label['prefix']['S'], service
 
 def get_xform_info(xformname):
+    if (xformname.find(':') != -1):
+        return get_xform_info_git(xformname)
+    else:
+        return get_xform_info_ddb(xformname)
+
+def get_xform_info_git(xformname):
+    dst_dir = tempfile.mkdtemp()
+    import git
+    git.Git(dst_dir).clone(xformname, '.')
+    with open(os.path.join(dst_dir, 'xformcode.py'), 'r') as xformcodefile:
+        xformcode = xformcodefile.read()
+    try:
+        with open(os.path.join(dst_dir, 'Dockerfile'), 'r') as dockerfile:
+            dockerfile_str = dockerfile.read()
+    except:
+        print('get_xform_info_git: No dockerfile. code=' + xformcode, flush=True)
+        return xformcode, None
+    else:
+        print('get_xform_info_git: dockerfile=' + dockerfile_str + ', code=' + xformcode, flush=True)
+        return xformcode, dockerfile_str
+
+def get_xform_info_ddb(xformname):
     tokfile = expanduser('~') + sep + '.infinstor' + sep + '/token'
     token, refresh_token, token_time, client_id, service, token_type = read_token_file(tokfile)
 
@@ -573,7 +595,16 @@ def run_script(transformation_string, **kwargs):
     if (verbose == True):
         print('run_script: transformSrc=' + transformSrc);
 
-    tree = ast.parse(transformSrc)
+    if (len(kwargs.items()) > 0):
+        tr_src = 'import sys\n'
+        lis = "['infin_transform'"
+        for key, value in kwargs.items():
+            lis = lis + ", '--" + key + "=" + value + "'"
+        tr_src = tr_src + 'sys.argv = ' + lis + "]\n" + transformSrc
+    else:
+        tr_src = transformSrc
+
+    tree = ast.parse(tr_src)
 
     compiledcode2 = compile(tree, "<string>", "exec")
 
@@ -863,25 +894,7 @@ def get_mlflow_run_artifacts_info(run_id):
 def generate_cache_key(input_data_spec, xformname):
     input_json_str = json.dumps(input_data_spec)
     input_json_bytes = input_json_str.encode('utf-8')
-    return hashlib.md5(input_json_bytes).hexdigest()
-    # if (input_data_spec['type'] == 'infinsnap'):
-    #     cache_key = 'xform/' + xformname + '/infinsnap/'\
-    #             + input_data_spec['time_spec'] + '/' + input_data_spec['bucketname']\
-    #             + '/' + input_data_spec['prefix']
-    # elif (input_data_spec['type'] == 'infinslice'):
-    #     cache_key = 'xform/' + xformname + '/infinslice/'\
-    #             + input_data_spec['time_spec'] + '/' + input_data_spec['bucketname']\
-    #             + '/' + input_data_spec['prefix']
-    # elif (input_data_spec['type'] == 'label'):
-    #     cache_key = 'xform/' + xformname + '/label/' + input_data_spec['label']
-    # elif (input_data_spec['type'] == 'mlflow-run-artifacts'):
-    #     cache_key = 'xform/' + xformname + '/run_id/' + input_data_spec['run_id']
-    # elif (input_data_spec['type'] == 'no-input-data'):
-    #     cache_key = 'xform/' + xformname + '/no-input-data/' \
-    #         +''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in range(8))
-    # else:
-    #     raise ValueError('Error. Unknown input_data_spec type ' + input_data_spec['type'])
-    # return cache_key
+    return 'xform/' + xformname + '/' + hashlib.md5(input_json_bytes).hexdigest()
 
 def generate_input_data_spec_string(input_data_spec):
     if (input_data_spec['type'] == 'infinsnap'):
@@ -904,7 +917,7 @@ def check_in_cache(input_data_spec, xformname):
     cache_key = generate_cache_key(input_data_spec, xformname)
 
     all_experiments = [exp.experiment_id for exp in MlflowClient().list_experiments()]
-    query = "params.cache_key = '" + cache_key + "'"
+    query = "tags.cache_key = '" + cache_key + "'"
     runs = MlflowClient().search_runs(experiment_ids=all_experiments,
             filter_string=query, run_view_type=ViewType.ACTIVE_ONLY)
     if (len(runs) == 0):
@@ -987,7 +1000,7 @@ def run_transform_inline(service_name, run_id, input_data_spec, xformname, **kwa
     else:
         cache_key, endpoint, prefix_in, prefix_trunc, bucketname, session, client, input_spec_string\
             = get_data_connection_details(service_name, input_data_spec, xformname)
-        log_param('cache_key', cache_key)
+        set_tag('cache_key', cache_key)
         log_param('input_data_spec', input_spec_string)
         xobjects = run_transform_string_inline(client, bucketname, prefix_in, prefix_trunc,
                 transform_string, **kwargs)
